#!/usr/bin/perl -w

use Fcntl ':flock';
use File::Temp qw/tmpnam/;
use Cwd qw(abs_path);

my $machine = `uname -n`;
$machine =~ s/[\r\n]//g;

my $port = shift;
if ($port) {
	if (($port !~ m/^\d+$/) or ($port > 65535)) {
		die "    Error: Invalid port '$port' specified!\n";
	}
}
else {
	$port = 8890;
}

my $sudo_cmd;
if ($> != 0) {
	$sudo_cmd = shift;
}

my $global_run = 0;
my $appchecker_dir = abs_path($0);
$appchecker_dir =~ s!/[^/]+/[^/]+$!!;
my $app_data;
if ($appchecker_dir eq '/opt/moblin/app-checker') {
	if (($> == 0) or $sudo_cmd) {
		$global_run = 1;
		$app_data = '/var/opt/moblin/app-checker';
	}
	else {
		$app_data = (getpwuid($>))[7].'/.moblin/app-checker';
	}
}
else {
	$app_data = $appchecker_dir;
}

print "    The port '$port' will be used by the AppChecker web-UI server.\n";
if ($sudo_cmd) {
	print "    The command '$sudo_cmd' will be used to gain root access.\n";
	print "    If you want to change this, run $0 <port> <sudo-command>\n\n";
}
else {
	print "    If you want to change this, run $0 <port>\n\n";
}

my $LOCK_FILE = "/tmp/app-checker-server.lock.$port";
if (-f $LOCK_FILE) {
	if (open(LOCKFILE, $LOCK_FILE)) {
		if (!flock(LOCKFILE, LOCK_EX | LOCK_NB)) {
			my $pid = <LOCKFILE>;
			$pid =~ s/^[LG]:(\d+)$/$1/;
			my $user = `ps -p $pid -o user=`;
			$user =~ s/[\r\n]//g;
			if ($user eq 'moblinappchecker') {
				$user = 'root';
			}
			die "    Error: An AppChecker instance is running on the port '$port' already!\n    (Started by user '$user'; PID: $pid.)\n\n    If you wish to use it, please open the URL http://$machine:$port/ in your browser.\n";
		}
		else {
			flock(LOCKFILE, LOCK_UN);
		}
		close(LOCKFILE);
	}
}

$appchecker_dir =~ s/([ ()\'])/\\$1/g;
my $cmd = "mkdir -p \"$app_data/results\" >/dev/null 2>&1; mkdir -p \"$app_data/log\" >/dev/null 2>&1;".($global_run ? " chown -R moblinappchecker:moblinappchecker $app_data;" : '')." $appchecker_dir/webui/app-checker-server.pl $port &";
if ($sudo_cmd) {
	$cmd = "$sudo_cmd '$cmd'";
}
my $res = system($cmd);
sleep(1);
if ($res != 0) {
	die "    Failed to start server.\n";
}
print "    Server started. Log file location:\n    $app_data/log/app-checker-server.log.$port\n";

# Try to open start page in a browser
print "\n    The start page should be opened in a browser shortly.\n    If it doesn't open, you can load it at http://$machine:$port/\n";

my $browser_not_found = 5;

# First, try the Portland's xdg-open util.
if (system('which xdg-open >/dev/null 2>&1') == 0) {
	my $err_file = tmpnam();
	system("xdg-open \"http://127.0.0.1:$port/\" >$err_file 2>&1");
	# An exit code of 0 indicates success.
	# 1 Error in command line syntax. 
	# 2 One of the files passed on the command line did not exist. 
	# 3 A required tool could not be found. 
	# 4 The action failed.
	$browser_not_found = $?;
	if ($browser_not_found) {
		system("cat $err_file");
	}
	unlink($err_file);
}
else {
	# Try to find a browser by it's name.
	foreach my $br_name qw(opera /usr/bin/mozilla-firefox firefox konqueror epiphany galeon mozilla) {
		if (system("which $br_name >/dev/null 2>&1") == 0) {
			system("$br_name \"http://127.0.0.1:$port/\" >/dev/null 2>&1 &");
			$browser_not_found = 0;
			last;
		}
	}
}

if ($browser_not_found) {
	print "\n    Could not find a web-browser.\n    Go to 'http://$machine:$port/' to configure and launch tests.\n";
}
