#!/usr/bin/python
#
# Copyright (C) 2009, Intel Corporation.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms and conditions of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307 USA.
#
# Authors:
#       Hao, Chen <hao.h.chen@intel.com>
#

# moblindesktopchk -j <journal dir> -d < .desktop file list> -i <icon list>

import os
import sys
import re
import getopt
import ConfigParser
import string
from stat import *

sys.path.append(os.path.dirname(sys.argv[0]) + '/../share/moblinisv/')
from tetj import *

version = '0.2'
journal_dir = ''
test_desktop_files = {}
test_icon_files_valid = []
test_icon_files_invalid = []
test_icon_files = []
icon_sizes_found = {'16x16':False, '32x32':False, '64x64':False, '128x128':False}
icon_valid_path = '/usr/share/icons/hicolor'


def usage():
    print sys.argv[0] + ' version ' + version
    print 'Usage: ' + sys.argv[0]
    print '       [-h (this message)]'
    print '       [-j <dir path for journal files>]'
    print '       [-d <.desktop list file>]'
    print '       [-i <icons list file>]'
    sys.exit(1)


# Import .desktop files into list
def import_desktop_files(desktop_list_file):
    global test_desktop_files
    try:
        f = open(desktop_list_file, 'r')
        regex_str = '^([^\t]+\.desktop)\t(.+)$'
        for file in f.read().splitlines():
            m = re.compile(regex_str).search(file)
            if m is not None:
                fname = m.group(1)
                if (fname not in test_desktop_files) and os.path.isfile(fname):
                    test_desktop_files[fname] = m.group(2)
        f.close()
    except IOError, (errno, strerror):
        print '%s: I/O error(%s): %s'% (desktop_list_file, errno, strerror)
        sys.exit(2)

# Import icon files into list
def import_icon_files(icon_list_file):
    global test_icon_files_valid, test_icon_files_invalid, test_icon_files
    try:
        f = open(icon_list_file, 'r')
        path_re = re.compile('^(/tmp/pkg-unpack\.\w+-\w+)?' + icon_valid_path + '/')
        for file in f.read().splitlines():
            if ((file[-4:] == '.svg') or (file[-4:] == '.png')) and os.path.isfile(file):
                if path_re.search(file) is None:
                    if file not in test_icon_files_invalid:
                        test_icon_files_invalid.append(file)
                else:
                    if file not in test_icon_files_valid:
                        test_icon_files_valid.append(file)
        f.close()
        test_icon_files = test_icon_files_valid + test_icon_files_invalid
    except IOError, (errno, strerror):
        print '%s: I/O error(%s): %s'% (icon_list_file, errno, strerror)
        sys.exit(2)

# Checks and stores size of the icon
def check_icon_size(icon_file):
    global icon_sizes_found
    basename = os.path.basename(icon_file)
    if icon_file[-4:] == '.svg':
        # SVG is scalable, consider all sizes present
        for size in icon_sizes_found:
            icon_sizes_found[size] = True
    elif icon_file[-4:] == '.png':
        # PNG: read dimensions from the file
        f = open(icon_file, 'rb')
        f.seek(0x10)
        width = int(f.read(4).encode('hex'), 16)
        height = int(f.read(4).encode('hex'), 16)
        f.close()
        type = '%sx%s' % (width, height)
        if type in icon_sizes_found:
            icon_sizes_found[type] = True

# Check if icon specified in .desktop file is mobline compliant
def check_icon_file(file, journal):
    # New icon check, clear the array of sizes found
    for size in icon_sizes_found:
        icon_sizes_found[size] = False

    icon_found = False
    info = ''
    path_info = ''
    if file == '':
        info = 'Empty icon name specified.'
    else:
        if file[0] == '/':
            # Abosulte path specified: search for it in any package or in the file system
            path_re = re.compile('^(/tmp/pkg-unpack\.\w+-\w+)?' + re.escape(file) + '$')
            for icon_file in test_icon_files:
                if path_re.search(icon_file) is not None:
                    icon_found = True
                    check_icon_size(icon_file)
                    break
        else:
            # Name or name+extension given: search all such icons in test_icon_files
            # First check all valid paths
            for icon_file in test_icon_files_valid:
                basename = os.path.basename(icon_file)
                if (file == basename) or (file == basename[:-4]):
                    icon_found = True
                    check_icon_size(icon_file)
            # If not all sizes were found, try images from invalid paths
            if False in icon_sizes_found.values():
                for icon_file in test_icon_files_invalid:
                    basename = os.path.basename(icon_file)
                    if (file == basename) or (file == basename[:-4]):
                        icon_found = True
                        check_icon_size(icon_file)
                        # TODO: Only complain if we found an image that changed some size in icon_sizes_found from False into True
                        path_info += 'Incompliant icon path \'%s\'\n' % icon_file

    # Check if all sizes were found
    ret = TETJ_FAIL
    if not icon_found:
        if info == '':
            info = 'Unable to find specified icon: %s' % file
    else:
        all_found = True
        info = "Icon file '%s' found" % file
        extra_info = ', but the following sizes are missing: '
        for size in icon_sizes_found:
            all_found = all_found and icon_sizes_found[size]
            if not icon_sizes_found[size]:
                extra_info += '%s ' % size
        if all_found:
            ret = TETJ_PASS
            info += ', all sizes requirement met.'
        else:
            info += extra_info

    journal.purpose_start('Field "Icon"')
    journal.testcase_info(0, 0, 0, info)
    journal.result(ret)
    journal.purpose_end()

    journal.purpose_start('Icon path check')
    if path_info != '':
        journal.testcase_info(0, 0, 0, path_info)
        journal.result(TETJ_FAIL)
    else:
        journal.result(TETJ_PASS)
    journal.purpose_end()

# Check each field in .desktop file
def check_fields_in_desktop_file(file):
    global journal_dir
    tmp_str = ''
    print 'TESTING: %s' % file

    # Create journal file
    journal_file = test_desktop_files[file]
    try:
        journal = Journal(journal_file, sys.argv[0])
    except  IOError:
        sys.stderr.write('Cannot create journal file: %s\n' % journal_file)
        sys.exit(1)
    journal.add_config('VSX_NAME=moblindesktopchk.py (%s)' % journal.machine)
    journal.config_end()
    journal.testcase_start(file)

    config = ConfigParser.RawConfigParser()
    config.read(file)
    # Section :  Desktop Entry
    journal.purpose_start('Section "Desktop Entry"')
    if config.has_section('Desktop Entry'):
        ret = TETJ_PASS
    else:
        ret = TETJ_FAIL
        tmp_str = 'Section "Desktop Entry" doesn\'t exist'
        journal.testcase_info(0, 0, 0, tmp_str)
    journal.result(ret)
    journal.purpose_end()

    # If the section exists, continue checking the fields
    if ret == TETJ_PASS:
        # Field :  Name
        journal.purpose_start('Field "Name"')
        ret = TETJ_FAIL
        try:
            field = 'Name'
            value = config.get('Desktop Entry', field)
            ret = TETJ_PASS
        except ConfigParser.NoOptionError:
            tmp_str = 'field "%s" doesn\'t exist' % field
            journal.testcase_info(0, 0, 0, tmp_str)
        journal.result(ret)
        journal.purpose_end()

        # Field :  Comment
        journal.purpose_start('Field "Comment"')
        ret = TETJ_FAIL
        try:
            field = 'Comment'
            value = config.get('Desktop Entry', field)
            ret = TETJ_PASS
        except ConfigParser.NoOptionError:
            tmp_str = 'field "%s" doesn\'t exist' % field
            journal.testcase_info(0, 0, 0, tmp_str)
        journal.result(ret)
        journal.purpose_end()

        # Field :  Exec
        journal.purpose_start('Field "Exec" or "Link"')
        ret = TETJ_FAIL
        try:
            field = 'Exec'
            value = config.get('Desktop Entry', field)
            ret = TETJ_PASS
        except ConfigParser.NoOptionError:
            try:
                field = 'Link'
                value = config.get('Desktop Entry', field)
                ret = TETJ_PASS
            except ConfigParser.NoOptionError:
                tmp_str = 'field "Exec" or "Link" doesn\'t exist'
                journal.testcase_info(0, 0, 0, tmp_str)
        journal.result(ret)
        journal.purpose_end()

        # Field :  Type
        journal.purpose_start('Field "Type"')
        ret = TETJ_FAIL
        try:
            field = 'Type'
            value = config.get('Desktop Entry', field)
            ret = TETJ_PASS
        except ConfigParser.NoOptionError:
            tmp_str = 'field "%s" doesn\'t exist' % field
            journal.testcase_info(0, 0, 0, tmp_str)
        journal.result(ret)
        journal.purpose_end()

        # Field :  Categories
        journal.purpose_start('Field "Categories"')
        ret = TETJ_FAIL
        try:
            field = 'Categories'
            value = config.get('Desktop Entry', field)
            ret = TETJ_PASS
        except ConfigParser.NoOptionError:
            tmp_str = 'field "%s" doesn\'t exist' % field
            journal.testcase_info(0, 0, 0, tmp_str)
        journal.result(ret)
        journal.purpose_end()

        # Field :  Icon
        try:
            field = 'Icon'
            value = config.get('Desktop Entry', field)
            check_icon_file(value, journal)
        except ConfigParser.NoOptionError:
            journal.purpose_start('Field "Icon"')
            tmp_str = 'field "%s" doesn\'t exist' % field
            journal.testcase_info(0, 0, 0, tmp_str)
            journal.result(TETJ_FAIL)
            journal.purpose_end()

    journal.testcase_end()
    journal.close()

def check_desktop_files():
    for file in test_desktop_files:
        check_fields_in_desktop_file(file)

if __name__=='__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'hj:d:i:')
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)

    for o, a in opts:
        if o == '-j':
            journal_dir = a
        elif o == '-d':
            import_desktop_files(a)
        elif o == '-i':
            import_icon_files(a)
        elif o == '-h':
            usage()

    check_desktop_files()
