# Moblin Application Checker
# Manual Tests Module (ManualTests.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package ManualTests;
use strict;
use Exporter;
use Common;

@ManualTests::ISA = 'Exporter';
@ManualTests::EXPORT = qw(
	%moblin_manual_tests @moblin_manual_tests_sort
	&read_manual_tests &read_fips
);

our @error_msgs = ();

# {ID}->{MESSAGE} == <description_text>
#     ->{OPTIONAL} == [1|0]
our %moblin_manual_tests = ();

# Sorting order for the questions
our @moblin_manual_tests_sort = ();

# Reads the ManualTests file containing all the manual tests and fills in
# the arrays %moblin_manual_tests and @moblin_manual_tests_sort
sub read_manual_tests($) {
	my ($data_path) = @_;
	my $mod_path = abs_path(__FILE__);
	$mod_path =~ s!^(.*)/[^/]+$!$1!;
	my $appchecker_path = $mod_path;
	$appchecker_path =~ s!^(.*)/[^/]+$!$1!;
	if (!-f $data_path) {
		$data_path = "$mod_path/ManualTests";
	}
	if (open(MANTESTS, $data_path)) {
		my $test_id;
		while (my $line = <MANTESTS>) {
			next if ($line =~ m/^#/);
			$line =~ s/[\r\n]//g;
			next if ($line eq '');
			if ($line =~ m/^\[(.+)\]$/) {
				$test_id = $1;
				push @moblin_manual_tests_sort, $test_id;
				$moblin_manual_tests{$test_id} = {'MESSAGE' => '', 'OPTIONAL' => 0};
			}
			elsif ($line =~ m/^MESSAGE (.*)$/) {
				if ($test_id) {
					my $msg = $1;
					$msg =~ s/%APPCHECKER_DIR%/$appchecker_path/g;
					$moblin_manual_tests{$test_id}->{'MESSAGE'} .= $msg;
				}
			}
			elsif ($line =~ m/^OPTIONAL ([01])$/) {
				if ($test_id) {
					$moblin_manual_tests{$test_id}->{'OPTIONAL'} = $1;
				}
			}
		}
		close(MANTESTS);
	}
}

# Reads all the FIPs from the testrun directory specified
# Returns hash:
# ->{<CATEGORY>}{TITLE} = <value>
#               {TESTCASES}{<ID>}{FILEPATH} = <path to file>
#                                {MESSAGE} = <message text>
sub read_fips($) {
	my ($testrun_dir) = @_;
	if (!$testrun_dir) {
		push @error_msgs, "Test run is undefined!";
		return undef;
	}

	my @fips_files;
	if (opendir(DIR, $testrun_dir)) {
		@fips_files = grep(/^.*\.FIPS$/, readdir(DIR));
		closedir(DIR);
	}
	else {
		push @error_msgs, "Test run directory cannot be opened: $!";
		return undef;
	}

	my $result = {};
	foreach my $filename (@fips_files) {
		my $fips_file = "$testrun_dir/$filename";
		my $info_file = $fips_file;
		$info_file =~ s/FIPS$/INFO/;
		my $test_file_id;
		my $test_file_path;

		if (open(INFO_FILE, $info_file)) {
			while (my $line = <INFO_FILE>) {
				if ($line =~ m/^TITLE:\s*(.*)/) {
					$test_file_path = $1;
				}
				elsif ($line =~ m/^JOURNAL_FILENAME:\s*(.*)/) {
					$test_file_id = $1;
				}
			}
			close(INFO_FILE);
			my $test_file_name = (($test_file_path =~ m!/([^/]+)$!) ? $1 : $test_file_path);
			$test_file_id =~ s/^.*($test_file_name\.\d+-\d+)\.journal$/$1/;
			# Make path to the file user-readable
			$test_file_path =~ s/^\|//;
			$test_file_path =~ s/\|$//;
			$test_file_path =~ s/\|/ =&gt; /g;
		}
		else {
			push @error_msgs, "File $info_file cannot be opened: $!";
		}
		if (open(FIPS_FILE, $fips_file)) {
			my $category;
			my $testcase;
			while (my $line = <FIPS_FILE>) {
				next if ($line =~ m/^#/);
				$line =~ s/[\r\n]//g;
				if ($line =~ m/^\[(.*)\]$/) {
					$category = $1;
				}
				elsif ($line =~ m/^CATEGORY_TITLE:\s*(.*)/) {
					if ($category) {
						$result->{$category}->{'TITLE'} = $1;
					}
				}
				elsif ($line =~ m/^TESTCASE\s+(.*)/) {
					if ($category) {
						$testcase = "$test_file_id.$1";
						$result->{$category}->{'TESTCASES'}->{$testcase}->{'FILEPATH'} = $test_file_path;
						$result->{$category}->{'TESTCASES'}->{$testcase}->{'MESSAGE'} = '';
					}
				}
				elsif ($line =~ m/^TESTCASE_END$/) {
					$testcase = undef;
				}
				else {
					if ($category and $testcase) {
						$result->{$category}->{'TESTCASES'}->{$testcase}->{'MESSAGE'} .= (($result->{$category}->{'TESTCASES'}->{$testcase}->{'MESSAGE'} eq '') ? '' : '<br />').$line;
					}
				}
			}
			close(FIPS_FILE);
		}
		else {
			push @error_msgs, "File $fips_file cannot be opened: $!";
		}
	}

	return $result;
}

1;
