#!/usr/bin/perl -w

# Moblin Application Checker
# Help Page Module (tests_help.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

print 'HTTP/1.0 200 OK' . CRLF;
print 'Content-type: text/html' . CRLF . CRLF;

print_header("Information", "help");

print <<DATA;
<a name="top"></a>
<p align="justify"><b>The Moblin Application Checker</b> provides a web interface for executing application compatibility tests.</p><br />
<table style="border: dashed 1px #2f6fab; padding: 5px; padding-right: 10px; white-space: nowrap; background-color: #f9f9f9;"><tr><td>
<h3>Help Contents</h3>
<ol>
  <li><a href="#conf">Application Check</a>
    <ul>
      <li><a href="#browse_component">Select Components Pseudo-Dialog</a></li>
    </ul>
  </li>
  <li><a href="#results">Result History page</a></li>
  <li><a href="#report">Analysis Results page</a>
    <ul>
      <li><a href="#res-distrs">Distribution Compatibility tab</a></li>
      <li><a href="#res-com">App Components tab</a></li>
      <li><a href="#res-libs">External Libraries tab</a></li>
      <li><a href="#res-ints">External Interfaces tab</a></li>
      <li><a href="#res-cert">Certification tab</a></li>
      <li><a href="#manual-tests">Manual Tests</a></li>
      <li><a href="#upload">Upload page</a></li>
    </ul>
  </li>
  <li><a href="#help">Help page</a></li>
  <li><a href="#about">About page</a></li>
  <li><a href="#admin">Administration page</a></li>
  <li><a href="#troubleshooting">Troubleshooting</a></li>
</ol></td></tr>
</table>
<br />

<a name="conf"></a><h1>Application Check</h1>
<p align="justify">
This page allows you to configure the test system. The following options are present:
</p>
<ul>
<li><a name="app_name"></a><b>Application name:</b><br /><span style="color: green;"><i>This field is mandatory.</i></span><br />Specify the name of the application you are going to test here. It can be chosen arbitrarily and will only be used in reports, temporary file names, etc.</li>
<li><a name="file_formats"></a><b>Application components:</b><br /><span style="color: green;"><i>This field is mandatory.</i></span><br />In this field, you can specify components of your application. You can specify:
  <ul>
    <li>Individual files</li>
    <li>Whole directories</li>
    <li>Installed packages prepended with <b>pkg:</b> (RPM or DEB, depending on your system)</li>
    <li>RPM and DEB package files</li>
    <li>TAR.GZ and TAR.BZ2 archives</li>
  </ul>
The archives and packages will be unpacked, and files of the following types will be tested:
  <ul>
    <li>ELF (executables and shared objects)</li>
    <li>Perl scripts</li>
    <li>Python scripts</li>
    <li>Shell scripts</li>
  </ul>
All other files not specified explicitly will be ignored when testing, to avoid overflowing the report with errors from every such file.<br />
<b>Note:</b> If you wish to avoid typing long names by hand, use the <b>Select Components&hellip;</b> button, which opens the <a href="#browse_component"><b>Select Components pseudo-dialog</b></a> for visually selecting the files and directories.</li>
</ul>
In the expandable <b>Additional options</b> block, you can find the following options:
<ul>
<li><b>Architecture:</b><br />Allows to select the machine architecture. NOTE: Only architectures that are compatible with the machine being tested are listed.<br />
<i>Currently unsupported.</i></li>
<li><b>Moblin Version:</b><br />This combo box specifies the Moblin version that will be used for additional tests for Moblin compliance.</li>
<!--<li><b>Moblin Profile:</b><br />Allows to select the Moblin profile for the Moblin version selected.</li>-->
<li><a name="comments"></a><b>Comments:</b><br />Here you may write comments with any additional information. These comments are stored in the execution report and shown on the <a href="#results"><b>Result History</b></a> page.</li>
</ul>
<p align="justify">
After you have configured the settings you may click the button <b>Run the Test</b> located at the bottom of the page.<br />
You can quickly clear all the fields by clicking the <b>Clear Form</b> button.
</p>

<a name="profile_management"></a><h3>User Profile Management</h3>
<p align="justify">
This section enables you to save/load all the settings listed above in different user profile files. To save the settings, just specify the new profile name in the edit box (or use the drop-down list to select the existing profile and overwrite it), then click the <b>Save</b> button. To load a previously saved profile, select it from the drop-down list and click <b>Load</b>. The <b>Delete</b> button allows you to delete old profiles that are no longer needed.<br />
<b>Caution:</b> The profile is not automatically saved when you run the tests! The settings are just stored in a temporary <i>session profile</i>, which is automatically removed when no longer used. So if you wish to use the same set of options later, please save it explicitly using the <b>Save</b> button.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="browse_component"></a><h2>Select Components Pseudo-Dialog</h2>
<p align="justify">
This is a special page (pseudo-dialog) designed to easily select application components. The standard <b>Browse</b> dialog provided by the browser cannot be used for this purpose for the following reasons:
</p>
<ul>
  <li>This dialog shows the file system of the local machine where the browser is opened. The Moblin Application Checker can only test applications located on the same machine with itself; so if you are working with the Moblin Application Checker remotely, the standard <b>Browse</b> will select the wrong files (local instead of remote).</li>
  <li>This dialog does not allow selecting directories.</li>
  <li>Due to safety reasons, the standard Input File HTML control can be handled only manually, by explicit user actions. Hence, it is not possible to restore its value when, for example, loading user profiles.</li>
  <li>This dialog is browser-dependent and has only fixed set of features; it cannot obtain additional functionality needed by the Moblin AppChecker.</li>
</ul>
<p align="justify">
The Select Components pseudo-dialog contains the following tabs:
</p>
<ul>
  <li>
    <b>Files and Directories</b><br />
    This tab allows navigation through the file system and selection of files and directories to check. The following elements are present:
    <ul>
      <li>Current path. You can click any part of it to jump quickly to the corresponding directory. The hard drive icon represents root of the file system.</li>
      <li>The <b>Show Hidden</b> check box, which allows you to show or hide files and directories starting with a period.</li>
      <li>List of files and subdirectories in the current directory. To open a subdirectory, just click the line with its name. For selecting files and directories to check, use check boxes to the left from the name column. All selected files/directories on any depth level are automatically remembered and will not be lost as you navigate the file system.</li>
    </ul>
    In case you earlier have selected an item in the current directory that is not currently present in the file system, you will be warned and asked to unselect the incorrect items (you can ignore this warning and leave the selection if the missing file will be present at the moment the test starts e.g., it will be compiled from source).
  </li>
  <li>
    <b>Installed Packages</b><br />
    This tab allows selection of installed packages (RPM or DEB, depending on your host system). For searching the package by name you can use the Filter edit box; the list of packages shown will be automatically contracted according to the filter text you enter.
  </li>
  <li>
    <b>All Selected</b><br />
    This tab contains all selected components in one plain list, so that you can quickly view all the items together without having to go through all intermediate subdirectories, and/or uncheck the unneeded ones.
  </li>
</ul>
<p align="justify">
At the bottom of the dialog, you can see the total number of the items currently selected (you can click this line for opening the <b>All Selected</b> tab), and the <b>Finish</b> button. Click this button to confirm your selection and close the dialog box. To return without saving the selection, just close the dialog window in your browser as usual.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="results"></a><h1>Result History</h1>
<p align="justify">
This page shows the table with the full list of the test\'s execution results. The table contains the following columns:
</p>
<ul>
  <li><b>Date/Time</b><br />The date and time are shown here, when the test is finished, as well as the name of the application tested. At the left edge of this column a status-icon is shown:
    <ul>
      <li><img src="images/result-pass.png" alt="pass" width="25" height="41" /> No compatibility problems were detected.</li>
      <li><img src="images/result-warn.png" alt="warn" width="25" height="41" /> Some compatibility problems were found.</li>
      <li><img src="images/result-fail.png" alt="fail" width="25" height="41" /> There are serious compatibility problems for each distribution known by the Application Checker.</li>
    </ul>
    <!--If the icon contains the "LSB" letters, it means that your application passed Moblin compliance tests and is eligible for Moblin certification.<br />-->
    In the cell&lsquo;s right-bottom corner there are several additional icons:
    <ul>
      <li><img src="images/res-distrs.png" alt="" width="16" height="16" /> Opens the <a href="#res-distrs"><b>Distribution Compatibility</b></a> tab of the <b>Analysis Results</b> page for this test run (clicking the date/time link does the same).</li>
      <li><img src="images/res-com.png" alt="" width="16" height="16" /> Opens the <a href="#res-com"><b>App Components</b></a> tab.</li>
      <li><img src="images/res-libs.png" alt="" width="16" height="16" /> Opens the <a href="#res-libs"><b>External Libraries</b></a> tab.</li>
      <li><img src="images/res-ints.png" alt="" width="16" height="16" /> Opens the <a href="#res-ints"><b>External Interfaces</b></a> tab.</li>
      <!--<li><img src="images/run_manual.png" alt="" width="16" height="16" /> This icon is present only when manual tests were not finished yet. Clicking on it redirects you to the <a href="#manual-tests"><b>Manual Tests</b></a> page.</li>-->
      <li><img src="images/res-cert.png" alt="" width="16" height="16" /> Opens the <a href="#res-cert"><b>Certification</b></a> tab.</li>
      <li><img src="images/upload_data.png" alt="" width="16" height="16" /> This icon allows you to <a href="#upload"><b>upload</b></a> information about the application to the <a href="http://www.linuxfoundation.org/" target="_blank">Linux Foundation</a> for publishing in the Applications section of the <a href="http://dev.linuxfoundation.org/moblin-navigator/browse/app.php" target="_blank">Moblin Navigator</a>. If you have already uploaded this information, the icon is not shown.</li>
    </ul>
  </li>
  <li><b>Comment</b><br />The comments entered on the <a href="#conf"><b>Application&nbsp;Check</b></a> page are shown here.</li>
</ul>
<p align="justify">
To remove the reports that are no longer needed, just mark them with the check boxes shown at the left edge of the corresponding lines and click the <b>Remove Selected Entries</b> button below the table.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="report"></a><h2>Analysis Results page</h2>
<p align="justify">
The <b>Analysis Results</b> page contains the compatibility report for the application tested. At the top there is a short summary indicating general information (such as number of distributions compatible with the applications). Below this are four tabs, each containing the detailed information concerning different aspects of compatibility (see description below).
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="res-distrs"></a><h2>Distribution Compatibility tab</h2>
<p align="justify">
This tab contains general information about compatibility of your application with most known distributions registered in the Moblin database. In this tab the table displayed indicates the summary status for each distribution. The status can be one of the following:
</p>
<ul>
  <li><font color="#008000"><b>OK</b></font> means that no compatibility problems were detected, and the application should launch on the distribution (however, this does not guarantee that the application will work correctly).</li>
  <li><font color="#c0a000"><b>WARNINGS</b></font> means that some non-critical problems were found, such as if the application requires some unknown libraries or interfaces. Also, this status is shown when the application requires some library which is missing in the distribution, but does not contain any of the interfaces required by the application (in this case the application can be just recompiled with this library excluded from the linker options).</li>
  <li><font color="#ff0000"><b>INCOMPATIBLE</b></font> is shown when there are serious compatibility problems, that is the application uses some libraries or interfaces that are not present in the distribution.</li>
</ul>
<p align="justify">
When you hover the mouse cursor above the status cell for any distribution, detailed information is shown to the right with specific numbers of required libraries and interfaces that cause problems. If you click on this cell, the table expands and shows the detailed information for all distributions together.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="res-com"></a><h2>App Components tab</h2>
<p align="justify">
The list of components of your applications is shown here. For each component displayed, its compatibility status is indicated in the following columns:
</p>
<ul>
  <li><b>Component</b><br />Component of the application</li>
  <li><b>Distribution Compatibility</b><br />Distribution compatibility status for this particular component (how many distributions are able to run it)</li>
  <li><b>Missing Libraries</b><br />Number of libraries that are used by the component and are missed in at least one of the distributions</li>
  <li><b>Missing Interfaces</b><br />Number of interfaces that are used by the component and are missed in at least one of the distributions</li>
  <li><b>Unneeded Libraries</b><br />The number of libraries that the component requires but does not use any interfaces from them</li>
  <li><b>Moblin Compatibility</b><br />Shows information about non-Moblin libraries and interfaces required by the component</li>
  <li><b>Comments</b><br />Provides additional information not covered by the previous columns</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>

<a name="res-libs"></a><h2>External Libraries tab</h2>
<p align="justify">
This tab contains detailed report about all the libraries required by your application. By clicking the <b>Select distributions&hellip;</b> link, you can open the list of distributions and select which of them are of interest for you. After clicking the <b>Apply</b> button, the table below will show information with regard to the distributions selected only.
</p>
<p align="justify">
The table itself consists of the following columns:
</p>
<ul>
  <li><b>Library</b><br />Name of the library</li>
  <li><b>Dependent Application Components</b><br />The list of the application components that require this library (click the link to see the component names and paths)</li>
  <li><b>Interfaces Actually Used</b><br />Number of interfaces exported by the library used by the application</li>
  <li><b>Presence in Distributions</b><br />The number of distributions that provide this library. Click the <b>details&hellip;</b> link for the detailed list</li>
  <li><b>In Moblin?</b><br />Indicates whether this library is included in the Moblin of the specified version</li>
  <li><b>More Info</b><br />Provides link to additional window with detailed information about the library</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>

<a name="res-ints"></a><h2>External Interfaces tab</h2>
<p align="justify">
In this tab the external interfaces are listed that are required by the application to run. The following columns are shown:
</p>
<ul>
  <li><b>Interface</b><br />The name of the interface</li>
  <li><b>Version</b><br />Version of the interface, if specified</li>
  <li><b>Dependent Application Components</b><br />The list of the application components that require this interface (click the link to see the component names and paths)</li>
  <li><b>In Libraries</b><br />Name of the library providing the interface</li>
  <li><b>Presence in Distributions</b><br />The number of distributions that provide this library. Click the <b>details&hellip;</b> link for the detailed list</li>
  <li><b>In Moblin?</b><br />This flag shows whether the interface is in the Moblin</li>
  <li><b>More Info</b><br />Here some additional information can be present about the interface or how to eliminate the Moblin specification incompliance</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>

<a name="res-cert"></a><h2>Certification tab</h2>
<p align="justify">
This page contains the detailed report about the test results. The section <b>Host Info</b> contains the information about the machine where the tests were executed. Below there is a combo-box allowing you to select a grouping of test results:
</p>
<ul>
  <li><b>By Component</b><br />
  This is the default setting. Below the expandable tree with all the files tested and the number of passed and failed tests for each file is shown. By clicking file name, you can quickly go to the report for this particular file.<br />
  <a name="No_result"></a>If <b>No result</b> for some item is shown, this means that the test journal was not found. Usually the reason is that the test failed to run or was interrupted during the execution.<br />
  The <b>Detailed Reports by Components</b> section contains complete list of problems for each file tested and summary information (test statistics, tool used for testing this file, etc.). The table consists of the following columns:
    <ul>
      <li><b>Category</b><br />
      The problem category. It describes what was tested and why the test failed.
      </li>
      <li><b>Type</b><br />
      Severity of the problem. <a href="#severity">See below</a> for detailed explanation.
      </li>
      <li><b>Element</b><br />
      Specific element that caused the test to fail (e.g., library or interface name, ELF section, etc.).
      </li>
      <li><b>Comments</b><br />
      Additional comments that can be useful in eliminating the problem.
      </li>
    </ul>
    Clicking the plus sign in the most-left column expands the line and displays additional information, such as messages reported by the test in the journal file.
  </li>
  <li><b>By Category</b><br />
  All test results for all files are grouped by category. Below is shown the list of all categories with total number of fails for each category. Each category name is a link to the detailed list of problems in this category.
  The <b>Detailed Reports by Category</b> section contains for each category its short description and the complete list of problems in this category (for all components). The table consists of the same columns as above, except that the <b>Category</b> column is replaced with the <b>Component</b> column containing the path/name of the file tested.
  </li>
</ul>
<p align="justify">
<a name="severity"></a>For each test case its severity is shown. The most common are the following:
</p>
<ul>
  <li><b>Failed</b> means that a severe problem occurred (it usually implies that the application is not Moblin compliant).</li>
  <li><b>Warning</b> means that this some non-critical problem occurred.</li>
  <li><a name="FIP"></a><b>FIP</b> stands for "<i>Further Information Provided</i>". These FIP results occur when the test suite is unable to determine whether a result is a PASS or a FAIL. For Moblin Certification you have to resolve each FIP case on the <a href="#conf"><b>Manual&nbsp;Tests</b></a> page and, if necessary, upload your comments (in plain text) along with the test journals.</li>
  <li><b>Notice</b> appears for purely informative messages.</li>
  <li><b>Untested</b> is a status for test cases reporting that the test was not performed for some reason (e.g., not implemented yet).</li>
</ul>
<p align="justify">
You can use the test report for applying for Moblin certification. To begin with the certification process, just click the <b>apply for certification</b> link. You will be redirected to the <a href="http://$cert_sys_host$cert_sys_base" target="_blank">Certification System</a>. If necessary, you will be asked to register, create your company, and create an entry for the product you are going to certify.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="manual-tests"></a><h2>Manual Tests</h2>
<p align="justify">
Some test cases cannot be performed automatically and need manual check. They are marked in the test report as <span title="Further Information Provided" style="border-bottom: 1px dashed gray; cursor: help;">FIP</span>s. If there is at least one FIP present in the test report, at the top of the page you will see a red warning that manual tests should be performed, with link to the Manual Tests page.
</p>
<p align="justify">
The list of manual tests is divided by categories. For each test a short description is shown and path to the specific file this particular test case concerns. You need to check whether the requirement is met and check the corresponding <b>Yes</b> or <b>No</b> radio-button. After all the test cases are completed, press the <b>Finish</b> button. The summary report will be re-generated and opened automatically, the former FIPs being transformed into Passes and Failes.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="upload"></a><h2>Upload page</h2>
<p align="justify">
This page allows you to send the information about the application to the Linux Foundation so that it can be added to the <a href="http://dev.linuxfoundation.org/moblin-navigator/browse/app.php" target="_blank">Moblin Navigator</a> (the Applications section). The following fields are present:
</p>
<ul>
  <li><b>Contact name</b><br />Name of the person uploading the data.</li>
  <li><b>E-mail</b><br />Contact E-mail (it will be used for notification about adding the application to the Moblin Navigator database).</li>
  <li><b>Application name</b><br />Name of the application. By default it contains the name specified on the <a href="#conf"><b>Application&nbsp;Check</b></a> page.</li>
  <li><b>Application version</b><br />Version of the application.</li>
  <li><b>Category</b><br />Select the category your application belongs to.</li>
  <li><b>User interface style</b><br />Specify if your application has graphical or console user interface.</li>
  <li><b>License type</b><br />Specify the license type your application is distributed under (Open Source or Proprietary).</li>
  <li><b>Application homepage</b><br />Homepage of the application (if any).</li>
  <li><b>Comments</b><br />If you have any comments, please write them here.</li>
</ul>
<p align="justify">
The mandatory fields are marked with red asterisks.
</p>
<p align="justify">
If you click the <b>Send Data</b> button, all the information collected will be uploaded automatically to the Linux Foundation FTP server. Alternatively, you can click <b>Save as&hellip;</b> to get all these data in a tarball and send it to the Linux Foundation manually (e.g. via E-mail).
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="help"></a><h1>Help</h1>
<p align="justify">
Contains the description of the web interface of the Moblin Application Checker (the page you are reading now).
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="about"></a><h1>About</h1>
<p align="justify">
Contains the general information about the Moblin Application Checker: its version, license, etc.
</p>
<p align="right"><a href="#top">Back to top</a></p>

<a name="admin"></a><h1>Administration</h1>
<p align="justify">
This page is designed for performing administrative tasks. Please be careful doing anything on this page!
</p>
<p align="justify">
Currently the following blocks are present:
</p>
<ul>
<li><b>Status:</b><br />Shows current status of the web-server and its PID. The button <b>Stop server</b> shuts the server down. Please note that you will not be able to use the web interface after that!<br />The link <b>View server log</b> allows you to open the web-server log file directly in the browser.</li>
<li><b>Distribution Data Management</b><br />
  This block is designed for managing updates of the distribution data used by the AppChecker.<br />
  The <b>Installed version</b> field shows the timestamp of the data snapshot installed in your system.<br />
  In the <b>Last check performed at</b> date and time is shown when you checked for the updates last time.<br />
  The button <b>Check for updates</b> allows you to check if there is updated version of the data available. In this case you will be suggested to perform the update. You need an active Internet connection for this.<br />
  If you connect to the Internet via a proxy server, you can specify it in the corresponding field in the expandable <b>Advanced</b> section.
</li>
</ul>
<p align="right"><a href="#top">Back to top</a></p>

<a name="troubleshooting"></a><h1>Troubleshooting</h1>
<p align="justify">
Please write to <a href="mailto:lsb-appcheck-support\@linuxfoundation.org">lsb-appcheck-support\@linuxfoundation.org</a> if you have any problems.
</p>
<p align="right"><a href="#top">Back to top</a></p>
DATA

print_footer();
